# -*- coding: utf-8 -*-
from AccessControl import Unauthorized
from copy import deepcopy
from plone import api
from plone.protect.interfaces import IDisableCSRFProtection
from plone.restapi.batching import HypermediaBatch
from plone.restapi.deserializer import json_body
from plone.restapi.search.utils import unflatten_dotted_dict
from plone.restapi.serializer.converters import json_compatible
from plone.restapi.services import Service
from zExceptions import BadRequest
from zope.component import getUtility
from zope.interface import alsoProvides
from zope.interface import implementer
from zope.publisher.interfaces import IPublishTraverse

import logging

logger = logging.getLogger(__name__)


class DataGet(Service):
    def reply(self):
        tool = getUtility(self.store)
        query = self.parse_query()
        batch = HypermediaBatch(self.request, tool.search(**query))
        data = {
            "@id": batch.canonical_url,
            "items": [self.expand_data(x) for x in batch],
            "items_total": batch.items_total,
        }
        links = batch.links
        if links:
            data["batching"] = links
        return data

    def expand_data(self, record):
        data = {k: json_compatible(v) for k, v in record.attrs.items()}
        data["id"] = record.intid
        return data

    def parse_query(self):
        query = deepcopy(self.request.form)
        query = unflatten_dotted_dict(query)
        res = {}
        if "sort_on" in query:
            res["sort_index"] = query["sort_on"]
            del query["sort_on"]
        if "sort_order" in query:
            order = query["sort_order"]
            reverse = True
            if order in ["asc", "ascending"]:
                reverse = False
            res["reverse"] = reverse
            del query["sort_order"]
        res["query"] = query
        return res

    def get_commented_obj(self, record):
        uid = record._attrs.get("uid", "")
        try:
            obj = api.content.get(UID=uid)
        except Unauthorized:
            return

        if not obj:
            return
        if not api.user.has_permission(
            "rer.customersatisfaction: Access Customer Satisfaction", obj=obj
        ):
            # user does not have that permission on object
            return
        return obj


class DataAdd(Service):
    def reply(self):
        alsoProvides(self.request, IDisableCSRFProtection)
        form_data = json_body(self.request)
        self.validate_form(form_data=form_data)
        data = self.extract_data(form_data=form_data)
        tool = getUtility(self.store)
        try:
            res = tool.add(data)
        except ValueError as e:
            self.request.response.setStatus(500)
            return dict(
                error=dict(
                    type="InternalServerError",
                    message=getattr(e, "message", e.__str__()),
                )
            )

        if res:
            return self.reply_no_content()

        self.request.response.setStatus(500)
        return dict(
            error=dict(
                type="InternalServerError",
                message="Unable to add. Contact site manager.",
            )
        )

    def validate_form(self, form_data):
        pass

    def extract_data(self, form_data):
        return json_body(self.request)


@implementer(IPublishTraverse)
class TraversableService(Service):
    """ Update an entry """

    def __init__(self, context, request):
        super(TraversableService, self).__init__(context, request)
        self.id = ""
        self.errors = {}

    def publishTraverse(self, request, id):
        # Consume any path segments after /@addons as parameters
        try:
            self.id = int(id)
        except ValueError:
            raise BadRequest("Id should be a number.")
        return self

    def reply(self):
        raise NotImplementedError


class DataUpdate(TraversableService):
    """ Update an entry """

    def reply(self):
        alsoProvides(self.request, IDisableCSRFProtection)
        if not self.id:
            raise BadRequest("Missing id")

        form_data = json_body(self.request)

        tool = getUtility(self.store)
        res = tool.update(id=self.id, data=form_data)
        if not res:
            return self.reply_no_content()
        if res.get("error", "") == "NotFound":
            raise BadRequest('Unable to find item with id "{}"'.format(self.id))
        self.request.response.setStatus(500)
        return dict(
            error=dict(
                type="InternalServerError",
                message="Unable to update item. Contact site manager.",
            )
        )


class DataDelete(TraversableService):
    def reply(self):
        alsoProvides(self.request, IDisableCSRFProtection)
        if not self.id:
            raise BadRequest("Missing id")
        tool = getUtility(self.store)
        res = tool.delete(id=self.id)
        if not res:
            return self.reply_no_content()
        if res.get("error", "") == "NotFound":
            raise BadRequest('Unable to find item with id "{}"'.format(self.id))
        self.request.response.setStatus(500)
        return dict(
            error=dict(
                type="InternalServerError",
                message="Unable to delete item. Contact site manager.",
            )
        )


class DataClear(Service):
    def reply(self):
        alsoProvides(self.request, IDisableCSRFProtection)
        tool = getUtility(self.store)
        tool.clear()
        return self.reply_no_content()



# -*- coding: utf-8 -*-
from plone import api
from Products.Five import BrowserView
from rer.customersatisfaction.interfaces import ICustomerSatisfactionStore
from zope.component import getUtility
from Products.CMFPlone.PloneBatch import Batch

import logging

logger = logging.getLogger(__name__)


class View(BrowserView):
    def get_data(self):
        tool = getUtility(ICustomerSatisfactionStore)
        query_vote = self.request.form.get("vote", "")
        query = {"uid": self.context.UID()}
        # if vote in ["ok", "nok"]:
        #     query["vote"] = vote
        search_results = tool.search(query=query)
        res = {
            "ok": 0,
            "ok_with_comments": 0,
            "nok": 0,
            "nok_with_comments": 0,
            "total": len(search_results),
            "comments": [],
        }
        for review in tool.search(query=query):
            data = self.format_data(review)
            vote = data.get("vote", "")
            comment = data.get("comment", "")
            if vote not in ["ok", "nok"]:
                continue
            res[vote] += 1
            if comment:
                res["{}_with_comments".format(vote)] += 1
                if query_vote and query_vote != vote:
                    continue
                res["comments"].append(data)
        b_size = self.request.form.get("b_size", 20)
        b_start = self.request.form.get("b_start", 0)
        res["comments"] = Batch(res["comments"], int(b_size), int(b_start))
        return res

    def format_data(self, item):
        uid = item._attrs.get("uid", "")
        res = {
            "title": item._attrs.get("title", ""),
            "date": item._attrs.get("date", ""),
            "vote": item._attrs.get("vote", ""),
            "comment": item._attrs.get("comment", ""),
        }

        obj = api.content.get(UID=uid)
        if obj:
            # can be changed
            res["title"] = obj.Title()
            res["url"] = obj.absolute_url()
        return res



# -*- coding: utf-8 -*-
from AccessControl import Unauthorized
from datetime import datetime
from plone import api
from plone.restapi.batching import HypermediaBatch
from plone.restapi.search.utils import unflatten_dotted_dict
from plone.restapi.serializer.converters import json_compatible
from rer.customersatisfaction.interfaces import ICustomerSatisfactionStore
from rer.customersatisfaction.restapi.services.common import DataGet
from six import StringIO
from zope.component import getUtility

import csv
import logging
import six

logger = logging.getLogger(__name__)


class CustomerSatisfactionGet(DataGet):
    """
    Called on context
    """

    def reply(self):
        if api.user.is_anonymous():
            raise Unauthorized
        results = self.get_data()
        batch = HypermediaBatch(self.request, results)
        data = {
            "@id": batch.canonical_url,
            "items": [self.fix_fields(x) for x in batch],
            "items_total": batch.items_total,
        }
        links = batch.links
        if links:
            data["batching"] = links
        return data

    def fix_fields(self, data):
        data["last_vote"] = json_compatible(data["last_vote"])
        return data

    def get_data(self):
        tool = getUtility(ICustomerSatisfactionStore)
        reviews = {}

        query = unflatten_dotted_dict(self.request.form)
        text = query.get("text", "")
        if text:
            query_res = tool.search(query={"text": text})
        else:
            query_res = tool.search()
        for review in query_res:
            uid = review._attrs.get("uid", "")
            date = review._attrs.get("date", "")
            vote = review._attrs.get("vote", "")
            if uid not in reviews:
                obj = self.get_commented_obj(record=review)
                if not obj and not api.user.has_permission(
                    "rer.customersatisfaction: Show Deleted Feedbacks"
                ):
                    # only manager can list deleted object's reviews
                    continue
                new_data = {
                    "ok": 0,
                    "nok": 0,
                    "comments": [],
                    "title": review._attrs.get("title", ""),
                    "uid": uid,
                    "review_ids": [],
                }
                if obj:
                    # can be changed
                    new_data["title"] = obj.Title()
                    new_data["url"] = obj.absolute_url()
                reviews[uid] = new_data
            data = reviews[uid]
            if vote in ["ok", "nok"]:
                data[vote] += 1
            comment = review._attrs.get("comment", "")
            if comment:
                data["comments"].append(
                    {"comment": comment, "date": json_compatible(date), "vote": vote}
                )
            if not data.get("last_vote", None):
                data["last_vote"] = date
            else:
                if data["last_vote"] < date:
                    data["last_vote"] = date

        result = list(reviews.values())
        sort_on = self.request.form.get("sort_on", "last_date")
        sort_order = self.request.form.get("sort_order", "desc")
        reverse = sort_order.lower() in ["desc", "descending", "reverse"]
        if sort_on in ["ok", "nok", "title", "last_vote"]:
            result = sorted(result, key=lambda k: k[sort_on], reverse=reverse)
        return result


class CustomerSatisfactionCSVGet(DataGet):
    """
    """

    type = "customer_satisfaction"

    def render(self):
        data = self.get_data()
        if isinstance(data, dict):
            if data.get("error", False):
                self.request.response.setStatus(500)
                return dict(
                    error=dict(
                        type="InternalServerError",
                        message="Unable export. Contact site manager.",
                    )
                )
        self.request.response.setHeader("Content-Type", "text/comma-separated-values")
        now = datetime.now()
        self.request.response.setHeader(
            "Content-Disposition",
            'attachment; filename="{type}_{date}.csv"'.format(
                type=self.type, date=now.strftime("%d%m%Y-%H%M%S")
            ),
        )
        self.request.response.write(data)

    def get_data(self):
        if api.user.is_anonymous():
            raise Unauthorized
        tool = getUtility(ICustomerSatisfactionStore)
        sbuf = StringIO()
        rows = []
        columns = [
            "title",
            "url",
            "vote",
            "comment",
            "date",
        ]
        for item in tool.search():
            obj = self.get_commented_obj(record=item)
            if not obj and not api.user.has_permission(
                "rer.customersatisfaction: Show Deleted Feedbacks"
            ):
                # only manager can list deleted object's reviews
                continue
            data = {}
            for k, v in item.attrs.items():
                if k not in columns:
                    continue
                if isinstance(v, list):
                    v = ", ".join(v)
                if isinstance(v, int):
                    v = str(v)
                val = json_compatible(v)
                if six.PY2:
                    val = val.encode("utf-8")
                data[k] = val
            if obj:
                data["url"] = obj.absolute_url()
            else:
                data["url"] = ""
            rows.append(data)
        writer = csv.DictWriter(sbuf, fieldnames=columns, delimiter=",")
        writer.writeheader()
        for row in rows:
            try:
                writer.writerow(row)
            except Exception as e:
                logger.exception(e)
                return {"error": True}
        res = sbuf.getvalue()
        sbuf.close()
        if six.PY2:
            return res
        return res.encode()



# -*- coding: utf-8 -*-
from repoze.catalog.catalog import Catalog
from repoze.catalog.indexes.field import CatalogFieldIndex
from repoze.catalog.indexes.text import CatalogTextIndex
from souper.interfaces import ICatalogFactory
from souper.soup import NodeAttributeIndexer
from souper.soup import NodeTextIndexer
from zope.interface import implementer


@implementer(ICatalogFactory)
class CustomerSatisfactionSoupCatalogFactory(object):
    def __call__(self, context):
        catalog = Catalog()
        text_indexer = NodeTextIndexer(["title"])
        catalog[u"text"] = CatalogTextIndex(text_indexer)
        vote_indexer = NodeAttributeIndexer("vote")
        catalog[u"vote"] = CatalogFieldIndex(vote_indexer)
        uid_indexer = NodeAttributeIndexer("uid")
        catalog[u"uid"] = CatalogFieldIndex(uid_indexer)
        date_indexer = NodeAttributeIndexer("date")
        catalog[u"date"] = CatalogFieldIndex(date_indexer)
        return catalog



# -*- coding: utf-8 -*-
from plone import api
from plone.app.upgrade.utils import installOrReinstallProduct
from Products.CMFPlone.interfaces import INonInstallable
from zope.interface import implementer

try:
    from collective.recaptcha.settings import IRecaptchaSettings  # noqa

    HAS_COLLECTIVE_RECAPTCHA = True
except ImportError:
    HAS_COLLECTIVE_RECAPTCHA = False


@implementer(INonInstallable)
class HiddenProfiles(object):
    def getNonInstallableProfiles(self):
        """Hide uninstall profile from site-creation and quickinstaller."""
        return [
            "rer.customersatisfaction:uninstall",
            "rer.customersatisfaction:collective_recaptcha",
        ]


def post_install(context):
    """Post install script"""
    if HAS_COLLECTIVE_RECAPTCHA:
        # install it and customize it.
        installOrReinstallProduct(api.portal.get(), "collective.recaptcha")
        context.runAllImportStepsFromProfile(
            "profile-rer.customersatisfaction:collective_recaptcha"
        )


def uninstall(context):
    """Uninstall script"""
    # Do something at the end of the uninstallation of this package.



# -*- coding: utf-8 -*-
from plone import api
from plone.protect.interfaces import IDisableCSRFProtection
from rer.customersatisfaction.interfaces import ICustomerSatisfactionStore
from rer.customersatisfaction.restapi.services.common import DataAdd
from rer.customersatisfaction.restapi.services.common import DataClear
from rer.customersatisfaction.restapi.services.common import DataDelete
from zExceptions import BadRequest
from zope.component import getUtility
from zope.interface import alsoProvides

import logging
import os
import requests

logger = logging.getLogger(__name__)


try:
    from collective.recaptcha.settings import IRecaptchaSettings

    HAS_COLLECTIVE_RECAPTCHA = True
except ImportError:
    HAS_COLLECTIVE_RECAPTCHA = False


class CustomerSatisfactionAdd(DataAdd):
    """
    Called on context
    """

    store = ICustomerSatisfactionStore

    def validate_form(self, form_data):
        """
        check all required fields and parameters
        """
        for field in ["vote"]:
            value = form_data.get(field, "")
            if not value:
                raise BadRequest("Campo obbligatorio mancante: {}".format(field))
            if value not in ["ok", "nok"]:
                raise BadRequest("Voto non valido: {}".format(value))
        self.check_recaptcha(form_data)

    def get_secret_key(self):
        if HAS_COLLECTIVE_RECAPTCHA:
            return api.portal.get_registry_record(
                "private_key", interface=IRecaptchaSettings
            )

        return os.environ.get("RECAPTCHA_PRIVATE_KEY", "")

    def check_recaptcha(self, form_data):
        if "g-recaptcha-response" not in form_data:
            raise BadRequest("Campo obbligatorio mancante: Non sono un robot")
        secret = self.get_secret_key()

        if not secret:
            logger.error(
                "Missing Recaptcha private key. Set it into collective.recaptcha "
                "control panel or in RECAPTCHA_PRIVATE_KEY env variable."
            )
            raise BadRequest("Chiave privata di Recaptcha non impostata.")
        payload = {
            "response": form_data["g-recaptcha-response"],
            "secret": secret,
        }
        response = requests.post(
            url="https://www.google.com/recaptcha/api/siteverify", data=payload
        )
        result = response.json()
        if not result.get("success", False):
            raise BadRequest("Validazione richiesta per il campo: Non sono un robot")
        return True

    def extract_data(self, form_data):
        data = super(CustomerSatisfactionAdd, self).extract_data(form_data)

        context_state = api.content.get_view(
            context=self.context,
            request=self.request,
            name=u"plone_context_state",
        )
        context = context_state.canonical_object()
        data["uid"] = context.UID()
        data["title"] = context.Title()
        if "g-recaptcha-response" in data:
            del data["g-recaptcha-response"]
        return data


class CustomerSatisfactionDelete(DataDelete):
    """"""

    store = ICustomerSatisfactionStore

    def publishTraverse(self, request, id):
        # Consume any path segments after /@addons as parameters
        self.id = id
        return self

    def reply(self):
        alsoProvides(self.request, IDisableCSRFProtection)
        if not self.id:
            raise BadRequest("Missing uid")
        tool = getUtility(self.store)
        reviews = tool.search(query={"uid": self.id})
        for review in reviews:
            res = tool.delete(id=review.intid)
            if not res:
                continue
            if res.get("error", "") == "NotFound":
                raise BadRequest('Unable to find item with id "{}"'.format(self.id))
            self.request.response.setStatus(500)
            return dict(
                error=dict(
                    type="InternalServerError",
                    message="Unable to delete item. Contact site manager.",
                )
            )
        return self.reply_no_content()


class CustomerSatisfactionClear(DataClear):
    """"""

    store = ICustomerSatisfactionStore



# -*- coding: utf-8 -*-
from rer.customersatisfaction.testing import (
    RER_CUSTOMERSATISFACTION_API_FUNCTIONAL_TESTING,
)
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from rer.customersatisfaction.interfaces import ICustomerSatisfactionStore
from plone.restapi.testing import RelativeSession
from zope.component import getUtility

import transaction
import unittest
import requests_mock
import os


class TestCustomerSatisfactionAdd(unittest.TestCase):

    layer = RER_CUSTOMERSATISFACTION_API_FUNCTIONAL_TESTING

    def setUp(self):
        os.environ["RECAPTCHA_PRIVATE_KEY"] = "foo"
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

        api.user.create(
            email="memberuser@example.com",
            username="memberuser",
            password="secret",
        )

        self.document = api.content.create(
            title="Document", container=self.portal, type="Document"
        )

        self.api_session = RelativeSession(self.portal_url)
        self.api_session.headers.update({"Accept": "application/json"})
        self.api_session.auth = (SITE_OWNER_NAME, SITE_OWNER_PASSWORD)
        self.anon_api_session = RelativeSession(self.portal_url)
        self.anon_api_session.headers.update({"Accept": "application/json"})

        self.url = "{}/@customer-satisfaction-add".format(self.document.absolute_url())
        transaction.commit()

    def tearDown(self):
        self.api_session.close()
        self.anon_api_session.close()

    def test_anon_can_post_data(self):
        # 400 because there are some missing fields
        self.assertEqual(self.anon_api_session.post(self.url, json={}).status_code, 400)

    @requests_mock.Mocker(real_http=True)
    def test_required_params(self, m):
        """
        email and channel are required.
        """
        self.assertEqual(self.anon_api_session.post(self.url).status_code, 400)

        # vote is required
        res = self.anon_api_session.post(self.url, json={})
        self.assertEqual(res.status_code, 400)
        self.assertEqual(res.json()["message"], "Campo obbligatorio mancante: vote")
        m.post(
            "https://www.google.com/recaptcha/api/siteverify",
            json={"success": False},
        )

        # captcha is required
        res = self.anon_api_session.post(self.url, json={"vote": "ok"})
        self.assertEqual(res.status_code, 400)
        self.assertEqual(
            res.json()["message"], "Campo obbligatorio mancante: Non sono un robot"
        )

        # captcha code is invalid
        m.post(
            "https://www.google.com/recaptcha/api/siteverify",
            json={"success": False},
        )
        res = self.anon_api_session.post(
            self.url, json={"vote": "ok", "g-recaptcha-response": "xyz"}
        )
        self.assertEqual(res.status_code, 400)
        self.assertEqual(
            res.json()["message"],
            "Validazione richiesta per il campo: Non sono un robot",
        )

        # captcha code is valid
        m.post(
            "https://www.google.com/recaptcha/api/siteverify",
            json={"success": True},
        )
        res = self.anon_api_session.post(
            self.url, json={"vote": "ok", "g-recaptcha-response": "xyz"}
        )
        self.assertEqual(res.status_code, 204)

    @requests_mock.Mocker(real_http=True)
    def test_validate_vote(self, m):
        # mock captcha verification
        m.post(
            "https://www.google.com/recaptcha/api/siteverify",
            json={"success": True},
        )
        resp = self.anon_api_session.post(
            self.url,
            json={"vote": 1, "comment": "i disagree", "g-recaptcha-response": "xyz"},
        )
        self.assertEqual(resp.status_code, 400)
        self.assertEqual(resp.json()["message"], "Voto non valido: 1")

    @requests_mock.Mocker(real_http=True)
    def test_correctly_save_data(self, m):
        # mock captcha verification
        m.post(
            "https://www.google.com/recaptcha/api/siteverify",
            json={"success": True},
        )
        self.anon_api_session.post(
            self.url,
            json={
                "vote": "ok",
                "comment": "i disagree",
                "g-recaptcha-response": "xyz",
            },
        )
        transaction.commit()
        tool = getUtility(ICustomerSatisfactionStore)
        self.assertEqual(len(tool.search()), 1)

    @requests_mock.Mocker(real_http=True)
    def test_store_only_known_fields(self, m):
        # mock captcha verification
        m.post(
            "https://www.google.com/recaptcha/api/siteverify",
            json={"success": True},
        )
        self.anon_api_session.post(
            self.url,
            json={
                "vote": "nok",
                "comment": "i disagree",
                "unknown": "mistery",
                "g-recaptcha-response": "xyz",
            },
        )
        transaction.commit()
        tool = getUtility(ICustomerSatisfactionStore)
        res = tool.search()
        self.assertEqual(len(res), 1)
        self.assertEqual(res[0]._attrs.get("unknown", None), None)
        self.assertEqual(res[0]._attrs.get("vote", None), "nok")
        self.assertEqual(res[0]._attrs.get("comment", None), "i disagree")

    @requests_mock.Mocker(real_http=True)
    def test_raise_error_if_private_key_not_set(self, m):
        """ """
        del os.environ["RECAPTCHA_PRIVATE_KEY"]

        # captcha code is valid
        m.post(
            "https://www.google.com/recaptcha/api/siteverify",
            json={"success": True},
        )
        res = self.anon_api_session.post(
            self.url, json={"vote": "ok", "g-recaptcha-response": "xyz"}
        )
        self.assertEqual(res.status_code, 400)



# -*- coding: utf-8 -*-
from Products.Five import BrowserView
from plone import api
from plone.memoize import ram
from plone.protect.authenticator import createToken
from time import time
from AccessControl import Unauthorized

import pkg_resources


JS_TEMPLATE = "{portal_url}/++plone++rer.customersatisfaction/react/dist/{env_mode}/{name}.js?v={version}"  # noqa
CSS_TEMPLATE = "{portal_url}/++plone++rer.customersatisfaction/react/dist/{env_mode}/{name}.css?v={version}"  # noqa


class View(BrowserView):
    """ """

    def __call__(self):
        if api.user.is_anonymous():
            raise Unauthorized
        return super(View, self).__call__()

    @ram.cache(lambda *args: time() // (60 * 60))
    def get_version(self):
        return pkg_resources.get_distribution("rer.customersatisfaction").version

    def get_env_mode(self):
        return (
            api.portal.get_registry_record("plone.resources.development")
            and "dev"  # noqa
            or "prod"  # noqa
        )

    def get_resource_js(self, name="history"):
        return JS_TEMPLATE.format(
            portal_url=api.portal.get().absolute_url(),
            env_mode=self.get_env_mode(),
            name=name,
            version=self.get_version(),
        )

    def get_resource_css(self, name="history"):
        return CSS_TEMPLATE.format(
            portal_url=api.portal.get().absolute_url(),
            env_mode=self.get_env_mode(),
            name=name,
            version=self.get_version(),
        )

    def get_token(self):
        return createToken()

    def can_delete(self):
        return (
            api.user.has_permission(
                "rer.customersatisfaction: Manage Customer Satisfaction"
            )
            and "true"  # noqa
            or "false"  # noqa
        )



# -*- coding: utf-8 -*-
from plone.app.layout.viewlets.common import ViewletBase
from plone import api


class CustomerSatisfactionViewlet(ViewletBase):
    """
    """

    def render(self):
        context_state = api.content.get_view(
            context=self.context,
            request=self.request,
            name=u"plone_context_state",
        )
        if context_state.canonical_object() == api.portal.get():
            return ""
        if not context_state.is_view_template():
            return ""
        return self.index()



# -*- coding: utf-8 -*-
from plone import api
from plone.app.layout.globals.interfaces import IBodyClassAdapter
from plone.dexterity.interfaces import IDexterityContent
from zope.component import adapter
from zope.interface import implementer
from rer.customersatisfaction.interfaces import IRerCustomersatisfactionLayer


@adapter(IDexterityContent, IRerCustomersatisfactionLayer)
@implementer(IBodyClassAdapter)
class CustomerSatisfactionBodyClasses(object):
    def __init__(self, context, request):
        self.context = context
        self.request = request

    def get_classes(self, template, view):
        """
        Add a custom class to body tag if we can vote
        """
        context_state = api.content.get_view(
            context=self.context,
            request=self.request,
            name=u"plone_context_state",
        )
        if context_state.canonical_object() == api.portal.get():
            return []
        if not context_state.is_view_template():
            return []
        return ["customer-satisfaction-enabled"]



# -*- coding: utf-8 -*-
from datetime import datetime
from plone import api
from rer.customersatisfaction.interfaces import ICustomerSatisfactionStore
from souper.soup import get_soup
from souper.soup import Record
from zope.interface import implementer

import logging
import six

logger = logging.getLogger(__name__)


@implementer(ICustomerSatisfactionStore)
class CustomerSatisfactionStore(object):
    """
    """

    fields = [
        "uid",
        "url",
        "title",
        "comment",
        "vote",
    ]
    text_index = "text"
    indexes = ["text", "vote", "uid"]
    keyword_indexes = []

    @property
    def soup(self):

        return get_soup("customer_satisfaction_soup", api.portal.get())

    def add(self, data):
        record = Record()
        for k, v in data.items():
            if k not in self.fields:
                logger.debug("[ADD] SKIP unkwnown field: {}".format(k))
                continue
            record.attrs[k] = v
        record.attrs["date"] = datetime.now()
        return self.soup.add(record)

    def length(self):
        return len([x for x in self.soup.data.values()])

    def search(self, query=None, sort_index="date", reverse=True):
        queries = []
        if query:
            queries = [
                self.parse_query_params(index, value)
                for index, value in query.items()
                if index in self.indexes and value
            ]
        if queries:
            return [
                x
                for x in self.soup.query(
                    " and ".join(queries),
                    sort_index=sort_index,
                    reverse=reverse,
                )
            ]
        # return all data
        records = self.soup.data.values()
        if sort_index == "date":
            return sorted(
                records,
                key=lambda k: k.attrs[sort_index] or None,
                reverse=reverse,
            )
        return sorted(
            records, key=lambda k: k.attrs[sort_index] or "", reverse=reverse
        )

    def parse_query_params(self, index, value):
        """
        """
        if index == self.text_index:
            return "'{}' in {}".format(value, self.text_index)
        elif index in self.keyword_indexes:
            if isinstance(value, list):
                return "{} in any({})".format(index, value)
            elif isinstance(value, six.text_type) or isinstance(value, str):
                return "{} in any('{}')".format(index, value)
        else:
            if isinstance(value, int):
                return "{} == {}".format(index, value)
            else:
                return "{} == '{}'".format(index, value)

    def get_record(self, id):
        if isinstance(id, six.text_type) or isinstance(id, str):
            try:
                id = int(id)
            except ValueError as e:
                logger.exception(e)
                return None
        try:
            return self.soup.get(id)
        except KeyError as e:
            logger.exception(e)
            return None

    def update(self, id, data):
        try:
            record = self.soup.get(id)
        except KeyError:
            logger.error('[UPDATE] item with id "{}" not found.'.format(id))
            return {"error": "NotFound"}
        for k, v in data.items():
            if k not in self.fields:
                logger.debug("[UPDATE] SKIP unkwnown field: {}".format(k))

            else:
                record.attrs[k] = v
        self.soup.reindex(records=[record])

    def delete(self, id):
        try:
            record = self.soup.get(id)
        except KeyError:
            logger.error(
                '[DELETE] Subscription with id "{}" not found.'.format(id)
            )
            return {"error": "NotFound"}
        del self.soup[record]

    def clear(self):
        self.soup.clear()



# -*- coding: utf-8 -*-
from plone.app.contenttypes.testing import PLONE_APP_CONTENTTYPES_FIXTURE
from plone.app.testing import (
    applyProfile,
    FunctionalTesting,
    IntegrationTesting,
    PloneSandboxLayer,
)
from plone.restapi.testing import PloneRestApiDXLayer
from plone.testing import z2

import collective.MockMailHost
import rer.customersatisfaction
import souper.plone
import plone.restapi

try:
    import collective.recaptcha

    HAS_COLLECTIVE_RECAPTCHA = True
except ImportError:
    HAS_COLLECTIVE_RECAPTCHA = False


class RerCustomersatisfactionLayer(PloneSandboxLayer):

    defaultBases = (PLONE_APP_CONTENTTYPES_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        # Load any other ZCML that is required for your tests.
        # The z3c.autoinclude feature is disabled in the Plone fixture base
        # layer.
        if HAS_COLLECTIVE_RECAPTCHA:
            self.loadZCML(package=collective.recaptcha)
        self.loadZCML(package=plone.restapi)
        self.loadZCML(package=rer.customersatisfaction)
        self.loadZCML(package=souper.plone)

    def setUpPloneSite(self, portal):
        applyProfile(portal, "rer.customersatisfaction:default")


RER_CUSTOMERSATISFACTION_FIXTURE = RerCustomersatisfactionLayer()


RER_CUSTOMERSATISFACTION_INTEGRATION_TESTING = IntegrationTesting(
    bases=(RER_CUSTOMERSATISFACTION_FIXTURE,),
    name="RerCustomersatisfactionLayer:IntegrationTesting",
)


RER_CUSTOMERSATISFACTION_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(RER_CUSTOMERSATISFACTION_FIXTURE,),
    name="RerCustomersatisfactionLayer:FunctionalTesting",
)


class RerCustomersatisfactionLayerApi(PloneRestApiDXLayer):

    defaultBases = (PLONE_APP_CONTENTTYPES_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        super(RerCustomersatisfactionLayerApi, self).setUpZope(
            app, configurationContext
        )
        if HAS_COLLECTIVE_RECAPTCHA:
            self.loadZCML(package=collective.recaptcha)
        self.loadZCML(package=plone.restapi)
        self.loadZCML(package=rer.customersatisfaction)
        self.loadZCML(package=souper.plone)

    def setUpPloneSite(self, portal):
        applyProfile(portal, "rer.customersatisfaction:default")


RER_CUSTOMERSATISFACTION_API_FIXTURE = RerCustomersatisfactionLayerApi()
RER_CUSTOMERSATISFACTION_API_INTEGRATION_TESTING = IntegrationTesting(
    bases=(RER_CUSTOMERSATISFACTION_API_FIXTURE,),
    name="RerCustomersatisfactionLayerApi:Integration",
)

RER_CUSTOMERSATISFACTION_API_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(RER_CUSTOMERSATISFACTION_API_FIXTURE, z2.ZSERVER_FIXTURE),
    name="RerCustomersatisfactionLayerApi:Functional",
)
